@@ -1,10 +1,10 @@
-#!/usr/bin/python2
-import random
-
+#!/usr/bin/python
 import uncrc32
 
-try: import pcap as pcapy
-except: import pcapy
+try:
+    import pcap as pcapy
+except ImportError:
+    import pcapy
 
 from impacket import ImpactPacket
 from impacket import ImpactDecoder
@@ -65,7 +65,7 @@ class Responder:
 
    def __init__(self, machine):
        self.machine = machine
-       print "Initializing %s" % self.__class__.__name__
+       print("Initializing %s" % self.__class__.__name__)
        self.initTemplate()
        self.initFingerprint()
 
@@ -79,12 +79,12 @@ class Responder:
              probe = self.templateClass(0, ['0.0.0.0',self.getIP()])
           self.template_onion = [probe.get_packet()]
           try:
-             while 1: self.template_onion.append(self.template_onion[-1].child())
+              while 1: self.template_onion.append (self.template_onion[-1].child ())
           except: pass
        
-          # print "Template: %s" % self.template_onion[O_ETH]
-          # print "Options: %r" % self.template_onion[O_TCP].get_padded_options()
-          # print "Flags: 0x%04x" % self.template_onion[O_TCP].get_th_flags()
+          # print("Template: %s" % self.template_onion[O_ETH])
+          # print("Options: %r" % self.template_onion[O_TCP].get_padded_options())
+          # print("Flags: 0x%04x" % self.template_onion[O_TCP].get_th_flags())
 
    def initFingerprint(self):
        if not self.signatureName:
@@ -103,7 +103,7 @@ class Responder:
 
    def process(self, in_onion):
        if not self.isMine(in_onion): return False
-       print "Got packet for %s" % self.__class__.__name__
+       print("Got packet for %s" % self.__class__.__name__)
 
        out_onion = self.buildAnswer(in_onion)
 
@@ -210,9 +210,7 @@ class ICMPResponder(IPResponder):
        if not IPResponder.isMine(self, in_onion): return False
        if len(in_onion) < 3: return False
 
-       return (
-           (in_onion[O_ICMP].protocol == ImpactPacket.ICMP.protocol) and
-           self.sameICMPTemplate(in_onion))
+       return (in_onion[O_ICMP].protocol == ImpactPacket.ICMP.protocol) and self.sameICMPTemplate(in_onion)
 
    def sameICMPTemplate(self, in_onion):
        t_ip           = self.template_onion[O_IP]
@@ -220,11 +218,11 @@ class ICMPResponder(IPResponder):
        t_icmp_datalen = self.template_onion[O_ICMP_DATA].get_size()
 
        return (
-          (t_ip.get_ip_tos() == in_onion[O_IP].get_ip_tos()) and
-          (t_ip.get_ip_df() == in_onion[O_IP].get_ip_df()) and
-          (t_icmp.get_icmp_type() == in_onion[O_ICMP].get_icmp_type()) and
-          (t_icmp.get_icmp_code() == in_onion[O_ICMP].get_icmp_code()) and
-          (t_icmp_datalen == in_onion[O_ICMP_DATA].get_size())
+               (t_ip.get_ip_tos () == in_onion[O_IP].get_ip_tos ()) and (
+                   t_ip.get_ip_df () == in_onion[O_IP].get_ip_df ()) and (
+                           t_icmp.get_icmp_type () == in_onion[O_ICMP].get_icmp_type ()) and (
+                           t_icmp.get_icmp_code () == in_onion[O_ICMP].get_icmp_code ()) and (
+                           t_icmp_datalen == in_onion[O_ICMP_DATA].get_size ())
        )
 
 class UDPResponder(IPResponder):
@@ -305,18 +303,13 @@ class TCPResponder(IPResponder):
        if not IPResponder.isMine(self, in_onion): return False
        if len(in_onion) < 3: return False
 
-       return (
-           in_onion[O_TCP].protocol == ImpactPacket.TCP.protocol and
-           self.sameTCPFlags(in_onion) and
-           self.sameTCPOptions(in_onion)
-       )
+       return (in_onion[O_TCP].protocol == ImpactPacket.TCP.protocol and self.sameTCPFlags (in_onion) and self.sameTCPOptions (
+           in_onion))
 
 class OpenTCPResponder(TCPResponder):
    def isMine(self, in_onion):
-       return (
-          TCPResponder.isMine(self, in_onion) and 
-          in_onion[O_TCP].get_SYN() and
-          self.machine.isTCPPortOpen(in_onion[O_TCP].get_th_dport()))
+       return (TCPResponder.isMine (self, in_onion) and in_onion[O_TCP].get_SYN () and self.machine.isTCPPortOpen (
+           in_onion[O_TCP].get_th_dport ()))
 
    def buildAnswer(self, in_onion):
        out_onion = TCPResponder.buildAnswer(self, in_onion)
@@ -357,14 +350,13 @@ class UDPCommandResponder(OpenUDPResponder):
        return self
 
    def isMine(self, in_onion):
-       return (
-          OpenUDPResponder.isMine(self, in_onion))# and 
+       return ( OpenUDPResponder.isMine(self, in_onion))# and
           #in_onion[O_UDP].get_uh_dport() == self.port)
 
    def buildAnswer(self, in_onion):
        cmd = in_onion[O_UDP_DATA].get_bytes().tostring()
        if cmd[:4] == 'cmd:': cmd = cmd[4:].strip()
-       print "Got command: %r" % cmd
+       print("Got command: %r" % cmd)
 
        if cmd == 'exit':
           from sys import exit
@@ -661,7 +653,7 @@ class NMAP2TCPResponder(TCPResponder):
           if opt == 'T':
              opt = TCPOption(TCPOption.TCPOPT_TIMESTAMP)  # default ts = 0, ts_echo = 0
              if options[i] == '1':  opt.set_ts(self.machine.getTCPTimeStamp())
-             if options[i+1] == '1': opt.set_ts_echo(0xffffffffL)
+             if options[i+1] == '1': opt.set_ts_echo(0xffffffff)
              tcp.add_option(opt)
              i += 2
           if opt == 'M':
@@ -766,7 +758,7 @@ class Machine:
        self.initSequenceGenerators()
        self.openTCPPorts = openTCPPorts
        self.openUDPPorts = openUDPPorts
-       print self
+       print(self)
 
    def openUDPPort(self, port):
        if self.isUDPPortOpen(port): return
@@ -793,16 +785,16 @@ class Machine:
 
    def initFingerprint(self, emmulating, nmapOSDB):
        fpm = os_ident.NMAP2_Fingerprint_Matcher('')
-       f = file(nmapOSDB, 'r')
+       f = open(nmapOSDB, 'r')
        for text in fpm.fingerprints(f):
            fingerprint = fpm.parse_fp(text)
            if fingerprint.get_id() == emmulating:
               self.fingerprint = fingerprint
               self.simplifyFingerprint()
-              # print fingerprint
+              # print(fingerprint)
               return
 
-       raise Exception, "Couldn't find fingerprint data for %r" % emmulating
+       raise Exception("Couldn't find fingerprint data for %r" % emmulating)
 
    def simplifyFingerprint(self):
        tests = self.fingerprint.get_tests()
@@ -854,8 +846,8 @@ class Machine:
            self.getIPID()
            self.getIPID_ICMP()
 
-       print "IP ID Delta: %d" % self.ip_ID_delta
-       print "IP ID ICMP Delta: %s" % self.ip_ID_ICMP_delta
+       print("IP ID Delta: %d" % self.ip_ID_delta)
+       print("IP ID ICMP Delta: %s" % self.ip_ID_ICMP_delta)
 
    def initTCPISNGenerator(self):
        # tcp_ISN and tcp_ISN_delta for TCP Initial sequence numbers
@@ -892,8 +884,8 @@ class Machine:
        # generate a few, so we don't start with 0 when we don't have to
        for i in range(10): self.getTCPSequence()
 
-       print "TCP ISN Delta: %f" % self.tcp_ISN_delta
-       print "TCP ISN Standard Deviation: %f" % self.tcp_ISN_stdDev
+       print("TCP ISN Delta: %f" % self.tcp_ISN_delta)
+       print("TCP ISN Standard Deviation: %f" % self.tcp_ISN_stdDev)
 
    def initTCPTSGenerator(self):
        # tcp_TS and tcp_TS_delta for TCP Time stamp generation
@@ -909,13 +901,13 @@ class Machine:
        # generate a few, so we don't start with 0 when we don't have to
        for i in range(10): self.getTCPTimeStamp()
 
-       print "TCP TS Delta: %f" % self.tcp_TS_delta
+       print("TCP TS Delta: %f" % self.tcp_TS_delta)
 
    def getIPID(self):
        answer = self.ip_ID
        self.ip_ID += self.ip_ID_delta
-       self.ip_ID %= 0x10000L
-       # print "IP ID: %x" % answer
+       self.ip_ID %= 0x10000
+       # print("IP ID: %x" % answer)
        return answer
 
    def getIPID_ICMP(self):
@@ -924,8 +916,8 @@ class Machine:
 
        answer = self.ip_ID_ICMP
        self.ip_ID_ICMP += self.ip_ID_ICMP_delta
-       self.ip_ID_ICMP %= 0x10000L
-       # print "---> IP ID: %x" % answer
+       self.ip_ID_ICMP %= 0x10000
+       # print("---> IP ID: %x" % answer)
        return answer
 
    def getTCPSequence(self):
@@ -933,22 +925,22 @@ class Machine:
        self.tcp_ISN_stdDev *= -1
        answer = int(int(answer/self.tcp_ISN_GCD) * self.tcp_ISN_GCD)
        self.tcp_ISN += self.tcp_ISN_delta
-       self.tcp_ISN %= 0x100000000L
-       # print "---> TCP Sequence: %d" % (answer % 0x100000000L)
-       return answer % 0x100000000L
+       self.tcp_ISN %= 0x100000000
+       # print("---> TCP Sequence: %d" % (answer % 0x100000000))
+       return answer % 0x100000000
 
    def getTCPTimeStamp(self):
        answer = int(round(self.tcp_TS))
        self.tcp_TS += self.tcp_TS_delta
-       self.tcp_TS %= 0x100000000L
-       # print "---> TCP Time Stamp: %x" % answer
+       self.tcp_TS %= 0x100000000
+       # print("---> TCP Time Stamp: %x" % answer)
        return answer
 
    def sendPacket(self, onion):
        if not onion: return
-       print "--> Packet sent:"
-       #print onion[0]
-       #print
+       print("--> Packet sent:")
+       #print(onion[0])
+       #print()
        self.pcap.sendpacket(onion[O_ETH].get_packet())
 
    def addResponder(self, aResponder):
@@ -964,7 +956,7 @@ class Machine:
           except:
              pass
 
-          #print "-------------- Received: ", in_onion[0]
+          #print("-------------- Received: ", in_onion[0])
           for r in self.responders:
               if r.process(in_onion): break
 
@@ -994,7 +986,7 @@ def main():
 
    from sys import argv, exit
    def usage():
-       print """
+       print("""
        if arg == '-h': usage()
        if arg == '--help': usage()
        if arg == '-f': Fingerprint = value
@@ -1006,12 +998,11 @@ def main():
    where:
        arg = argv[i]
        value = argv[i+1]
-       """
+       """)
        exit()
 
    global Fingerprint, IFACE, MAC, IP, nmapOSDB
-   for i in xrange(len(argv)):
-       arg = argv[i]
+   for i, arg in enumerate(argv):
        try: value = argv[i+1]
        except: value = None
        if arg == '-h': usage()
@@ -1022,8 +1013,8 @@ def main():
        if arg == '-i': IFACE = value
        if arg == '-d': nmapOSDB = value
 
-   print "Emulating: %r" % Fingerprint
-   print "at %s / %s / %s" % (IFACE, MAC, IP)
+   print("Emulating: %r" % Fingerprint)
+   print("at %s / %s / %s" % (IFACE, MAC, IP))
    machine = Machine(
        Fingerprint,
        IFACE,
@@ -1031,7 +1022,7 @@ def main():
        MAC,
        OPEN_TCP_PORTS,
        OPEN_UDP_PORTS,
-       nmapOSDB = nmapOSDB)
+       nmapOSDB=nmapOSDB)
 
    initResponders(machine)
    machine.initGenericResponders()
