@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python
 # SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
@@ -14,7 +14,8 @@
 #  DCE/RPC.
 # TODO: 
 # [ ] Check errors
-
+from __future__ import division
+from __future__ import print_function
 import sys
 import argparse
 import logging
@@ -24,7 +25,7 @@ from impacket.examples import logger
 from impacket import version
 from impacket.dcerpc.v5 import transport, scmr
 from impacket.dcerpc.v5.ndr import NULL
-from impacket.crypto import *
+from impacket.crypto import encryptSecret
 
 
 class SVCCTL:
@@ -47,7 +48,7 @@ class SVCCTL:
 
     def run(self, remoteName, remoteHost):
 
-        stringbinding = 'ncacn_np:%s[\pipe\svcctl]' % remoteName
+        stringbinding = r'ncacn_np:%s[\pipe\svcctl]' % remoteName
         logging.debug('StringBinding %s'%stringbinding)
         rpctransport = transport.DCERPCTransportFactory(stringbinding)
         rpctransport.set_dport(self.__port)
@@ -90,70 +91,70 @@ class SVCCTL:
         elif self.__action == 'CONFIG':
             logging.info("Querying service config for %s" % self.__options.name)
             resp = scmr.hRQueryServiceConfigW(rpc, serviceHandle)
-            print "TYPE              : %2d - " % resp['lpServiceConfig']['dwServiceType'],
+            print("TYPE              : %2d - " % resp['lpServiceConfig']['dwServiceType'], end=' ')
             if resp['lpServiceConfig']['dwServiceType'] & 0x1:
-                print "SERVICE_KERNEL_DRIVER ",
+                print("SERVICE_KERNEL_DRIVER ", end=' ')
             if resp['lpServiceConfig']['dwServiceType'] & 0x2:
-                print "SERVICE_FILE_SYSTEM_DRIVER ",
+                print("SERVICE_FILE_SYSTEM_DRIVER ", end=' ')
             if resp['lpServiceConfig']['dwServiceType'] & 0x10:
-                print "SERVICE_WIN32_OWN_PROCESS ",
+                print("SERVICE_WIN32_OWN_PROCESS ", end=' ')
             if resp['lpServiceConfig']['dwServiceType'] & 0x20:
-                print "SERVICE_WIN32_SHARE_PROCESS ",
+                print("SERVICE_WIN32_SHARE_PROCESS ", end=' ')
             if resp['lpServiceConfig']['dwServiceType'] & 0x100:
-                print "SERVICE_INTERACTIVE_PROCESS ",
-            print ""
-            print "START_TYPE        : %2d - " % resp['lpServiceConfig']['dwStartType'],
+                print("SERVICE_INTERACTIVE_PROCESS ", end=' ')
+            print("")
+            print("START_TYPE        : %2d - " % resp['lpServiceConfig']['dwStartType'], end=' ')
             if resp['lpServiceConfig']['dwStartType'] == 0x0:
-                print "BOOT START"
+                print("BOOT START")
             elif resp['lpServiceConfig']['dwStartType'] == 0x1:
-                print "SYSTEM START"
+                print("SYSTEM START")
             elif resp['lpServiceConfig']['dwStartType'] == 0x2:
-                print "AUTO START"
+                print("AUTO START")
             elif resp['lpServiceConfig']['dwStartType'] == 0x3:
-                print "DEMAND START"
+                print("DEMAND START")
             elif resp['lpServiceConfig']['dwStartType'] == 0x4:
-                print "DISABLED"
+                print("DISABLED")
             else:
-                print "UNKNOWN"
+                print("UNKNOWN")
 
-            print "ERROR_CONTROL     : %2d - " % resp['lpServiceConfig']['dwErrorControl'],
+            print("ERROR_CONTROL     : %2d - " % resp['lpServiceConfig']['dwErrorControl'], end=' ')
             if resp['lpServiceConfig']['dwErrorControl'] == 0x0:
-                print "IGNORE"
+                print("IGNORE")
             elif resp['lpServiceConfig']['dwErrorControl'] == 0x1:
-                print "NORMAL"
+                print("NORMAL")
             elif resp['lpServiceConfig']['dwErrorControl'] == 0x2:
-                print "SEVERE"
+                print("SEVERE")
             elif resp['lpServiceConfig']['dwErrorControl'] == 0x3:
-                print "CRITICAL"
+                print("CRITICAL")
             else:
-                print "UNKNOWN"
-            print "BINARY_PATH_NAME  : %s" % resp['lpServiceConfig']['lpBinaryPathName'][:-1]
-            print "LOAD_ORDER_GROUP  : %s" % resp['lpServiceConfig']['lpLoadOrderGroup'][:-1]
-            print "TAG               : %d" % resp['lpServiceConfig']['dwTagId']
-            print "DISPLAY_NAME      : %s" % resp['lpServiceConfig']['lpDisplayName'][:-1]
-            print "DEPENDENCIES      : %s" % resp['lpServiceConfig']['lpDependencies'][:-1]
-            print "SERVICE_START_NAME: %s" % resp['lpServiceConfig']['lpServiceStartName'][:-1]
+                print("UNKNOWN")
+            print("BINARY_PATH_NAME  : %s" % resp['lpServiceConfig']['lpBinaryPathName'][:-1])
+            print("LOAD_ORDER_GROUP  : %s" % resp['lpServiceConfig']['lpLoadOrderGroup'][:-1])
+            print("TAG               : %d" % resp['lpServiceConfig']['dwTagId'])
+            print("DISPLAY_NAME      : %s" % resp['lpServiceConfig']['lpDisplayName'][:-1])
+            print("DEPENDENCIES      : %s" % resp['lpServiceConfig']['lpDependencies'][:-1])
+            print("SERVICE_START_NAME: %s" % resp['lpServiceConfig']['lpServiceStartName'][:-1])
         elif self.__action == 'STATUS':
-            print "Querying status for %s" % self.__options.name
+            print("Querying status for %s" % self.__options.name)
             resp = scmr.hRQueryServiceStatus(rpc, serviceHandle)
-            print "%30s - " % self.__options.name,
+            print("%30s - " % self.__options.name, end=' ')
             state = resp['lpServiceStatus']['dwCurrentState']
             if state == scmr.SERVICE_CONTINUE_PENDING:
-               print "CONTINUE PENDING"
+               print("CONTINUE PENDING")
             elif state == scmr.SERVICE_PAUSE_PENDING:
-               print "PAUSE PENDING"
+               print("PAUSE PENDING")
             elif state == scmr.SERVICE_PAUSED:
-               print "PAUSED"
+               print("PAUSED")
             elif state == scmr.SERVICE_RUNNING:
-               print "RUNNING"
+               print("RUNNING")
             elif state == scmr.SERVICE_START_PENDING:
-               print "START PENDING"
+               print("START PENDING")
             elif state == scmr.SERVICE_STOP_PENDING:
-               print "STOP PENDING"
+               print("STOP PENDING")
             elif state == scmr.SERVICE_STOPPED:
-               print "STOPPED"
+               print("STOPPED")
             else:
-               print "UNKNOWN"
+               print("UNKNOWN")
         elif self.__action == 'LIST':
             logging.info("Listing services available on target")
             #resp = rpc.EnumServicesStatusW(scManagerHandle, svcctl.SERVICE_WIN32_SHARE_PROCESS )
@@ -161,25 +162,25 @@ class SVCCTL:
             #resp = rpc.EnumServicesStatusW(scManagerHandle, serviceType = svcctl.SERVICE_FILE_SYSTEM_DRIVER, serviceState = svcctl.SERVICE_STATE_ALL )
             resp = scmr.hREnumServicesStatusW(rpc, scManagerHandle)
             for i in range(len(resp)):
-                print "%30s - %70s - " % (resp[i]['lpServiceName'][:-1], resp[i]['lpDisplayName'][:-1]),
+                print("%30s - %70s - " % (resp[i]['lpServiceName'][:-1], resp[i]['lpDisplayName'][:-1]), end=' ')
                 state = resp[i]['ServiceStatus']['dwCurrentState']
                 if state == scmr.SERVICE_CONTINUE_PENDING:
-                   print "CONTINUE PENDING"
+                   print("CONTINUE PENDING")
                 elif state == scmr.SERVICE_PAUSE_PENDING:
-                   print "PAUSE PENDING"
+                   print("PAUSE PENDING")
                 elif state == scmr.SERVICE_PAUSED:
-                   print "PAUSED"
+                   print("PAUSED")
                 elif state == scmr.SERVICE_RUNNING:
-                   print "RUNNING"
+                   print("RUNNING")
                 elif state == scmr.SERVICE_START_PENDING:
-                   print "START PENDING"
+                   print("START PENDING")
                 elif state == scmr.SERVICE_STOP_PENDING:
-                   print "STOP PENDING"
+                   print("STOP PENDING")
                 elif state == scmr.SERVICE_STOPPED:
-                   print "STOPPED"
+                   print("STOPPED")
                 else:
-                   print "UNKNOWN"
-            print "Total Services: %d" % len(resp)
+                   print("UNKNOWN")
+            print("Total Services: %d" % len(resp))
         elif self.__action == 'CREATE':
             logging.info("Creating service %s" % self.__options.name)
             scmr.hRCreateServiceW(rpc, scManagerHandle, self.__options.name + '\x00', self.__options.display + '\x00',
@@ -246,7 +247,7 @@ if __name__ == '__main__':
     if sys.stdout.encoding is None:
         # Output is redirected to a file
         sys.stdout = codecs.getwriter('utf8')(sys.stdout)
-    print version.BANNER
+    print(version.BANNER)
 
     parser = argparse.ArgumentParser(add_help = True, description = "Windows Service manipulation script.")
 
@@ -352,7 +353,7 @@ if __name__ == '__main__':
     services = SVCCTL(username, password, domain, options, int(options.port))
     try:
         services.run(remoteName, options.target_ip)
-    except Exception, e:
+    except Exception as e:
         if logging.getLogger().level == logging.DEBUG:
             import traceback
             traceback.print_exc()
