@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python
 # SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
@@ -17,6 +17,8 @@
 # Reference for:
 #  DCOM
 #
+from __future__ import division
+from __future__ import print_function
 import argparse
 import sys
 import os
@@ -27,7 +29,7 @@ from impacket import version
 from impacket.dcerpc.v5.dtypes import NULL
 from impacket.dcerpc.v5.dcom import wmi
 from impacket.dcerpc.v5.dcomrt import DCOMConnection
-from impacket.dcerpc.v5.rpcrt import RPC_C_AUTHN_LEVEL_PKT_PRIVACY, RPC_C_AUTHN_LEVEL_PKT_INTEGRITY, RPC_C_AUTHN_LEVEL_NONE
+from impacket.dcerpc.v5.rpcrt import RPC_C_AUTHN_LEVEL_PKT_PRIVACY, RPC_C_AUTHN_LEVEL_PKT_INTEGRITY
 
 if __name__ == '__main__':
     import cmd
@@ -40,12 +42,12 @@ if __name__ == '__main__':
             self.intro = '[!] Press help for extra shell commands'
 
         def do_help(self, line):
-            print """
+            print("""
      lcd {path}                 - changes the current local directory to {path}
      exit                       - terminates the server process (and this session)
      describe {class}           - describes class
      ! {cmd}                    - executes a local shell cmd
-     """ 
+     """) 
 
         def do_shell(self, s):
             os.system(s)
@@ -58,7 +60,7 @@ if __name__ == '__main__':
                 iObject, _ = self.iWbemServices.GetObject(sClass)
                 iObject.printInformation()
                 iObject.RemRelease()
-            except Exception, e:
+            except Exception as e:
                 if logging.getLogger().level == logging.DEBUG:
                     import traceback
                     traceback.print_exc()
@@ -66,7 +68,7 @@ if __name__ == '__main__':
 
         def do_lcd(self, s):
             if s == '':
-                print os.getcwd()
+                print(os.getcwd())
             else:
                 os.chdir(s)
     
@@ -77,21 +79,21 @@ if __name__ == '__main__':
                     pEnum = iEnum.Next(0xffffffff,1)[0]
                     record = pEnum.getProperties()
                     if printHeader is True:
-                        print '|',
+                        print('|', end=' ')
                         for col in record:
-                            print '%s |' % col,
-                        print
+                            print('%s |' % col, end=' ')
+                        print()
                         printHeader = False
-                    print '|', 
+                    print('|', end=' ') 
                     for key in record:
                         if type(record[key]['value']) is list:
                             for item in record[key]['value']:
-                                print item,
-                            print ' |',
+                                print(item, end=' ')
+                            print(' |', end=' ')
                         else:
-                            print '%s |' % record[key]['value'],
-                    print 
-                except Exception, e:
+                            print('%s |' % record[key]['value'], end=' ')
+                    print() 
+                except Exception as e:
                     if logging.getLogger().level == logging.DEBUG:
                         import traceback
                         traceback.print_exc()
@@ -109,7 +111,7 @@ if __name__ == '__main__':
                 iEnumWbemClassObject = self.iWbemServices.ExecQuery(line.strip('\n'))
                 self.printReply(iEnumWbemClassObject)
                 iEnumWbemClassObject.RemRelease()
-            except Exception, e:
+            except Exception as e:
                 logging.error(str(e))
          
         def emptyline(self):
@@ -120,7 +122,7 @@ if __name__ == '__main__':
 
     # Init the example's logger theme
     logger.init()
-    print version.BANNER
+    print(version.BANNER)
 
     parser = argparse.ArgumentParser(add_help = True, description = "Executes WQL queries and gets object descriptions "
                                                                     "using Windows Management Instrumentation.")
@@ -201,15 +203,14 @@ if __name__ == '__main__':
             shell.cmdloop()
         else:
             for line in options.file.readlines():
-                print "WQL> %s" % line,
+                print("WQL> %s" % line, end=' ')
                 shell.onecmd(line)
 
         iWbemServices.RemRelease()
         dcom.disconnect()
-    except Exception, e:
+    except Exception as e:
         logging.error(str(e))
         try:
             dcom.disconnect()
         except:
             pass
-
