@@ -9,16 +9,17 @@
 # Description:
 #   Transport implementations for the DCE/RPC protocol.
 #
+from __future__ import division
+from __future__ import print_function
 
-import re
-import socket
 import binascii
 import os
+import re
+import socket
 
-from impacket.smbconnection import smb, SMBConnection
-from impacket import nmb
 from impacket import ntlm
 from impacket.dcerpc.v5.rpcrt import DCERPCException, DCERPC_v5, DCERPC_v4
+from impacket.smbconnection import SMBConnection
 
 
 class DCERPCStringBinding:
@@ -66,12 +67,15 @@ class DCERPCStringBinding:
 
 def DCERPCStringBindingCompose(uuid=None, protocol_sequence='', network_address='', endpoint='', options=[]):
     s = ''
-    if uuid: s += uuid + '@'
+    if uuid:
+        s += uuid + '@'
     s += protocol_sequence + ':'
-    if network_address: s += network_address
+    if network_address:
+        s += network_address
     if endpoint or options:
         s += '[' + endpoint
-        if options: s += ',' + ','.join(options)
+        if options:
+            s += ',' + ','.join(options)
         s += ']'
 
     return s
@@ -138,15 +142,15 @@ class DCERPCTransport:
         self.set_credentials('','')
 
     def connect(self):
-        raise RuntimeError, 'virtual function'
+        raise RuntimeError('virtual function')
     def send(self,data=0, forceWriteAndx = 0, forceRecv = 0):
-        raise RuntimeError, 'virtual function'
+        raise RuntimeError('virtual function')
     def recv(self, forceRecv = 0, count = 0):
-        raise RuntimeError, 'virtual function'
+        raise RuntimeError('virtual function')
     def disconnect(self):
-        raise RuntimeError, 'virtual function'
+        raise RuntimeError('virtual function')
     def get_socket(self):
-        raise RuntimeError, 'virtual function'
+        raise RuntimeError('virtual function')
 
     def get_connect_timeout(self):
         return self.__connect_timeout
@@ -223,8 +227,10 @@ class DCERPCTransport:
         self._TGT      = TGT
         self._TGS      = TGS
         if lmhash != '' or nthash != '':
-            if len(lmhash) % 2:     lmhash = '0%s' % lmhash
-            if len(nthash) % 2:     nthash = '0%s' % nthash
+            if len(lmhash) % 2:
+                lmhash = '0%s' % lmhash
+            if len(nthash) % 2:
+                nthash = '0%s' % nthash
             try: # just in case they were converted already
                self._lmhash = binascii.unhexlify(lmhash)
                self._nthash = binascii.unhexlify(nthash)
@@ -256,7 +262,7 @@ class UDPTransport(DCERPCTransport):
             af, socktype, proto, canonname, sa = socket.getaddrinfo(self.getRemoteHost(), self.get_dport(), 0, socket.SOCK_DGRAM)[0]
             self.__socket = socket.socket(af, socktype, proto)
             self.__socket.settimeout(self.get_connect_timeout())
-        except socket.error, msg:
+        except socket.error as msg:
             self.__socket = None
             raise DCERPCException("Could not connect: %s" % msg)
 
@@ -297,7 +303,7 @@ class TCPTransport(DCERPCTransport):
         try:
             self.__socket.settimeout(self.get_connect_timeout())
             self.__socket.connect(sa)
-        except socket.error, msg:
+        except socket.error as msg:
             self.__socket.close()
             raise DCERPCException("Could not connect: %s" % msg)
         return 1
@@ -305,7 +311,7 @@ class TCPTransport(DCERPCTransport):
     def disconnect(self):
         try:
             self.__socket.close()
-        except socket.error, msg:
+        except socket.error:
             self.__socket = None
             return 0
         return 1
@@ -324,7 +330,7 @@ class TCPTransport(DCERPCTransport):
 
     def recv(self, forceRecv = 0, count = 0):
         if count:
-            buffer = ''
+            buffer = b''
             while len(buffer) < count:
                buffer += self.__socket.recv(count-len(buffer))
         else:
